
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "authors": [
  "Padraic Shafer"
 ],
 "date": "2021-03-16T07:11:44-0700",
 "dirty": false,
 "error": null,
 "full-revisionid": "02f2f88927818f6a56f311b9d230be8938ae8f17",
 "version": "0.9.0"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)



# -*- coding: utf-8 -*-

import sip
sip.setapi('QVariant', 2)	# Call this before referencing QtCore
sip.setapi('QString', 2)	# Call this before referencing QtCore

# from PyQt4 import QtGui, QtCore
# from PyQt4.QtCore import Qt, QString, QVariant, QRect, QRectF, QSize, QPoint
from PyQt4.QtCore import Qt, QRect, QRectF, QSize, QPoint
from PyQt4.QtCore import QObject, SIGNAL, SLOT, QCoreApplication
from PyQt4.QtCore import QAbstractTableModel, QModelIndex
from PyQt4.QtGui import QApplication, QMainWindow, QTextEdit, QLabel, QStyle
from PyQt4.QtGui import QStyledItemDelegate, QStyleOptionViewItemV4
from PyQt4.QtGui import QTextDocument, QPushButton, QStyleOptionButton
from PyQt4.QtGui import QStyleFactory, QGraphicsProxyWidget, QCheckBox
from PyQt4.QtGui import QSizePolicy, QFileDialog, QPalette, QColor
from PyQt4.QtGui import QHeaderView
# from PyQt4.QtGui import QCommonStyle, QCleanlooksStyle, QPlastiqueStyle
# from PyQt4.QtGui import QMacStyle, QWindowsStyle, QWindowsXPStyle
import pyqtgraph as pg
import als.liam.fitsViewerGui as fitsViewerGui
from als.liam.fitsParams import fitsParamsModel
from als.liam.fitsParams import qtFloatDelegate, qtHtmlDelegate
from als.liam.fitsParams import qtDateDelegate, qtTimeDelegate
from als.liam.fitsParams import fitsParamsLeftColModel, fitsParamsRightColModel

from PIL import Image
from numpy import nan, float32, sqrt, round, floor, array, log10
from numpy.random import random_integers, randint
from astropy.io import fits
import pandas as pd

import os
import sys
from als.milo.qimage import Diffractometer402, CcdImageFromFITS

try:
     #_fromUtf8 = QString.fromUtf8
    pass
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QApplication.translate(context, text, disambig)


pg.setConfigOptions(imageAxisOrder="row-major")


def show_motor_params(primary_hdu):
	# print 'Beamline Energy   : ' + str(primary_hdu['Beamline Energy']) + ' eV'
	print 'Mono Energy   : ' + str(primary_hdu['Mono Energy']) + ' eV'
	print 'Bottom Rotary Seal: ' + str(primary_hdu['Bottom Rotary Seal']) + ' deg'
	print 'Top Rotary Seal   : ' + str(primary_hdu['Top Rotary Seal']) + ' deg'
	print 'Flip              : ' + str(primary_hdu['Flip']) + ' deg'
	print 'Twice Top Offset  : ' + str(primary_hdu['Twice Top Offset']) + ' deg'
	
	# energy = primary_hdu['Beamline Energy']
	energy = primary_hdu['Mono Energy']
	bottom = primary_hdu['Bottom Rotary Seal']
	top = primary_hdu['Top Rotary Seal']
	flip = primary_hdu['Flip']
	offset_top = primary_hdu['Twice Top Offset'] / 2
	offset_flip = 0
	offset_ccd = -18.68
	
	if ('Top Offset' in primary_hdu):
		offset_top = primary_hdu['Top Offset']
		print 'Top Offset        : ' + str(offset_top) + ' deg'
	if ('Flip Offset' in primary_hdu):
		offset_flip = primary_hdu['Flip Offset']
		print 'Flip Offset       : ' + str(offset_flip) + ' deg'
	if ('CCD Offset' in primary_hdu):
		offset_ccd = primary_hdu['CCD Offset']
		print 'CCD Offset        : ' + str(offset_ccd) + ' deg'
		
	wavelength = 1239.842 / energy
	twotheta = bottom - offset_ccd
	truetop = top - offset_top
	incidence = bottom - truetop
	chi = flip - offset_flip
	
	print ''
	print 'Wavelength : ' + str(wavelength) + ' nm'
	print 'Detector   : ' + str(twotheta) + ' deg'
	print 'Incidence  : ' + str(incidence) + ' deg'
	print 'Chi        : ' + str(chi) + ' deg'


def load_image(filename):
	hdulist = fits.open(filename)
# 	print "We opened an image. It's filename is ", filename
# 	print "here's some info"
# 	print hdulist.info()
	
	# show_motor_params(hdulist[0].header)
	
	# let's assume, the last entry is the image.
	return (hdulist[0].header, hdulist[-1])
	

class FitsViewerApp(QMainWindow, fitsViewerGui.Ui_MainWindow):
	def __init__(self, parent=None):
		super(FitsViewerApp, self).__init__(parent)
		self.setupUi(self)
		
		self.setWindowTitle(
			_translate("MainWindow", "FITS data viewer", None))
		
		glImage = self.glFitsImage
		
		glBoxImageTitle = (0, 0, 1, 3)
		glBoxImageSubtitle = (1, 0, 1, 3)
		glBoxPlotRows = (2, 0)
		glBoxImageView = (2, 1)
		glBoxLutHistog = (2, 2)
		glBoxPlotCols = (3, 1)
		glBoxColorScaling = (3, 2)
		
		self.txtImageTitle = pg.LabelItem(
			"Click 'Load data file' button to begin...", size='24pt')
		self.txtImageSubtitle = pg.LabelItem("Default data displayed")
		glImage.addItem(self.txtImageTitle, *glBoxImageTitle)
		glImage.addItem(self.txtImageSubtitle, *glBoxImageSubtitle)
		self.txtImageTitle.setSizePolicy(
			QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
		self.txtImageSubtitle.setSizePolicy(
			QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
		
		self.image_array = randint(512, 2**16, size=(512, 512))
		image_array = self.image_array
		(imgRows, imgCols) = image_array.shape
		
		self.imv = glImage.addViewBox(*glBoxImageView)
		imv = self.imv
		# imv.invertY(True)		# Need to invert if image is not FITS
		img = pg.ImageItem(image_array, border=0.5)
		self.image = img
		self.image.setCursor(Qt.CrossCursor)
		imv.addItem(img)
		imv.setAspectLocked()
# 		imv.setLimits(
# 			maxXRange=imgCols, maxYRange=imgRows, 
# 			xMin=0, xMax=imgCols, yMin=0, yMax=imgRows, 
# 			)
		self.linRegRows = pg.LinearRegionItem(
			values=[0, 0], 
			orientation=pg.LinearRegionItem.Horizontal, 
			brush=(0, 0, 255, 64), 
			# pen='b', 
			bounds=(0, imgRows), 
			)
		self.linRegCols = pg.LinearRegionItem(
			values=[0, 0], 
			orientation=pg.LinearRegionItem.Vertical, 
			brush=(255, 255, 0, 64), 
			# pen = 'y', 
			bounds=(0, imgCols), 
			)
		imv.addItem(self.linRegRows)
		imv.addItem(self.linRegCols)
		# imv.setPredefinedGradient("flame")
		# imv.setLookupTable(xxx)
		# vbLutHistog = glImage.addViewBox(0, 1)
		self.lutHistog = pg.HistogramLUTItem(img)
		lutHistog = self.lutHistog
		# lutHistog.setImageItem(img)
		lutHistog.plot.rotate(-90)
		lutHistog.plot.setLogMode(False, True)
		lutHistog.plot.rotate(90)
		# lutHistog.sigLevelsChanged.connect(self.onImageLevelsChanged)
		lutHistog.sigLevelChangeFinished.connect(self.onImageLevelsChanged)
		lutHistog.gradient.loadPreset("flame")
		glImage.addItem(lutHistog, *glBoxLutHistog)
		# viewLutHistog = glImage.addViewBox(0, 2)
		# viewLutHistog.addItem(lutHistog)
		self.plotFitsRows = glImage.addPlot(*glBoxPlotRows)
		self.plotFitsCols = glImage.addPlot(*glBoxPlotCols)
		self.plotFitsRowsData = self.plotFitsRows.plot(
			image_array.mean(axis=1), 
			fillLevel=image_array.min(), 
			# -image_array.mean(axis=1), 
			# fillLevel = -(image_array.min()), 
			# fillLevel = (-image_array.astype(int)).max(), 
			fillBrush='y')
		self.plotFitsRowsData.scale(-1, 1)
		self.plotFitsRowsData.rotate(90)
		self.plotFitsRows.setYLink(imv)
		self.plotFitsRows.invertX(True)
		# self.plotFitsRows.invertY(True)
		self.plotFitsColsData = self.plotFitsCols.plot(
			image_array.mean(axis=0), 
			fillLevel=image_array.min(), 
			fillBrush='b')
		self.plotFitsCols.setXLink(imv)
		# self.plotFitsRows.rotate(90)
		glImage.ci.layout.setColumnMaximumWidth(glBoxPlotRows[1], 100)
		glImage.ci.layout.setRowMaximumHeight(glBoxPlotCols[0], 100)
		
		glImageScale = pg.GraphicsLayout()
		
		proxy = QGraphicsProxyWidget()
		self.chkAutoScale = QCheckBox('Auto scale')
		self.chkAutoScale.setChecked(True)
		self.chkAutoScale.stateChanged.connect(self.onAutoScaleChanged)
# 		self.chkAutoScale.setSizePolicy(
# 			QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
		proxy.setWidget(self.chkAutoScale)
		glImageScale.addItem(proxy, 0, 0)
		
		proxy = QGraphicsProxyWidget()
		# self.chkLogScale = QCheckBox('Log scale', self)
		self.chkLogScale = QCheckBox('Log scale')
		self.chkLogScale.setChecked(False)
		self.chkLogScale.stateChanged.connect(self.onLogScaleChanged)
# 		self.chkLogScale.setSizePolicy(
# 			QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
		proxy.setWidget(self.chkLogScale)
		glImageScale.addItem(proxy, 1, 0)
		
		glImageScale.setSizePolicy(
			QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
		
		glImage.addItem(glImageScale, *glBoxColorScaling)
		
# 		SLOT('onLinRegColsChanged(QObject)')
# 		SLOT('onLinRegRowsChanged(QObject)')
		
		self.linRegCols.sigRegionChanged.connect(self.onLinRegColsChanged)
		self.linRegRows.sigRegionChanged.connect(self.onLinRegRowsChanged)
# 		QObject.connect(
# 			# self.linRegCols, SIGNAL('sigRangeChanged(QObject)'), 
# 			self.linRegCols, SIGNAL('sigRangeChanged'), 
# 			self.onLinRegColsChanged)
# 		QObject.connect(
# 			# self.linRegRows, SIGNAL('sigRangeChanged(QObject)'), 
# 			self.linRegRows, SIGNAL('sigRangeChanged'), 
# 			self.onLinRegRowsChanged)
		
		imv.autoRange()
		
		self.spinDataFrameNum.setMinimum(0)
		self.spinDataFrameNum.setMaximum(0)
		
		self.image_cursor_label = None
		img.scene().sigMouseMoved.connect(self.onMouseMoved)
		
		self.btnSelectDataFile.clicked.connect(self.selectDataFile)
		self.btnReloadDataFile.clicked.connect(self.loadDataFile)
		
		self.spinDataFrameNum.valueChanged.connect(self.updateDataFrame)
		
		self.btnFirstDataFrame.clicked.connect(self.gotoFirstDataFrame)
		self.btnPrevDataFrame.clicked.connect(self.gotoPrevDataFrame)
		self.btnNextDataFrame.clicked.connect(self.gotoNextDataFrame)
		self.btnLastDataFrame.clicked.connect(self.gotoLastDataFrame)
		
		# Project directory is parent of "als/liam/"
		self.project_directory = os.path.dirname(
			os.path.dirname(
				os.path.dirname(
					os.path.abspath(__file__) 
					) 
				)
			)
		self.data_directory = "{}/test_data/".format(self.project_directory)
		self.working_directory = str(self.data_directory)
		self.data_pathnames = []
		self.data_frame_num = 0
		# self.selectDataFile()
		
# 		palSelectButton = QPalette()
# 		palSelectButton.setColor(QPalette.Button, QColor(Qt.yellow))
# 		self.btnSelectDataFile.setAutoFillBackground(True)
# 		self.btnSelectDataFile.setPalette(palSelectButton)

		self.cbPauseParamUpdates.setChecked(False)
		self.cbPauseParamUpdates.stateChanged.connect(
			self.onPauseFitsParamsUpdatesChanged)
		
		self.model_fits_params_left = fitsParamsLeftColModel()
		
		self.viewFitsParamsLeft.setModel(self.model_fits_params_left)
		self.viewFitsParamsLeft.setItemDelegateForColumn(
			0, 
			qtHtmlDelegate(self) )
		self.viewFitsParamsLeft.setItemDelegateForColumn(
			1, 
			qtFloatDelegate(self, decimals=3) )
		self.viewFitsParamsLeft.setItemDelegateForColumn(
			2, 
			qtHtmlDelegate(self) )
			
		self.viewFitsParamsLeft.resizeColumnsToContents()
		
		self.viewFitsParamsLeft.horizontalHeader().setResizeMode(
			1, QHeaderView.Stretch)
		self.viewFitsParamsLeft.horizontalHeader().setDefaultAlignment(
			Qt.AlignRight)
		self.viewFitsParamsLeft.horizontalHeader().hide()
		self.viewFitsParamsLeft.verticalHeader().hide()
# 		self.viewFitsParamsLeft.setHorizontalScrollBarPolicy(
# 			Qt.ScrollBarAlwaysOff)
# 		self.viewFitsParamsLeft.setVerticalScrollBarPolicy(
# 			Qt.ScrollBarAlwaysOff)
		# self.viewFitsParamsLeft.adjustSize()
		
		fitsParamsRightColModel
		
		self.model_fits_params_right = fitsParamsRightColModel()
		
		self.viewFitsParamsRight.setModel(self.model_fits_params_right)
		self.viewFitsParamsRight.setItemDelegateForColumn(
			0, 
			qtHtmlDelegate(self) )
		self.viewFitsParamsRight.setItemDelegateForColumn(
			1, 
			qtFloatDelegate(self, decimals=3) )
		self.viewFitsParamsRight.setItemDelegateForColumn(
			2, 
			qtHtmlDelegate(self) )
			
		self.viewFitsParamsRight.resizeColumnsToContents()
		
		self.viewFitsParamsRight.horizontalHeader().setResizeMode(
			1, QHeaderView.Stretch)
		self.viewFitsParamsRight.horizontalHeader().setDefaultAlignment(
			Qt.AlignRight)
		self.viewFitsParamsRight.horizontalHeader().hide()
		self.viewFitsParamsRight.verticalHeader().hide()
		
		self.viewFitsParamsRight.setSpan(0, 0, 1, 3)
		self.viewFitsParamsRight.setSpan(1, 0, 1, 3)
		self.viewFitsParamsRight.setItemDelegateForRow(
			0, 
			qtDateDelegate(self) )
		self.viewFitsParamsRight.setItemDelegateForRow(
			1, 
			qtTimeDelegate(self) )
		
		# self.setStyleSheet(".sub { vertical-align: sub }")
		# self.setStyleSheet(".sup { vertical-align: super }")
	
	def onLinRegColsChanged(self, linReg):
		
		# self.plotFitsRows.setYRange(linReg.getRegion())
		image_array = self.image_array
		(minCol, maxCol) = linReg.getRegion()
		(minCol, maxCol) = (int(round(minCol)), int(round(maxCol)))
		if (minCol == maxCol):
			minCol = 0
			maxCol = image_array.shape[1]
		# print "onLinRegRowsChanged:", linReg.getRegion()
		self.plotFitsRowsData.setData(image_array[:, minCol:maxCol].mean(axis=1))
		# self.plotFitsRowsData.setData(-image_array[:, minCol:maxCol].mean(axis=1))
	
	def onLinRegRowsChanged(self, linReg):
		
		# self.plotFitsCols.setXRange(linReg.getRegion())
		image_array = self.image_array
		(minRow, maxRow) = linReg.getRegion()
		(minRow, maxRow) = (int(round(minRow)), int(round(maxRow)))
		if (minRow == maxRow):
			minRow = 0
			maxRow = image_array.shape[0]
		# print "onLinRegColsChanged:", linReg.getRegion()
		self.plotFitsColsData.setData(image_array[minRow:maxRow].mean(axis=0))
	
	def onLogScaleChanged(self, state):
		
		log_mode = (state & Qt.Checked)
		
		lutHistog_levels = self.lutHistog.getLevels()
# 		if self.lutHistog.axis.logMode:
# 			lutHistog_levels = 10 ** array(lutHistog_levels)
		
		image_array = self.image_array
		if log_mode:
			image_array = log10(self.image_array)
			lutHistog_levels = log10(lutHistog_levels)
		else:
			lutHistog_levels = 10 ** array(lutHistog_levels)
		fill_level = image_array.min()
		self.plotFitsRows.getAxis("bottom").setLogMode(log_mode)
		self.plotFitsCols.getAxis("left").setLogMode(log_mode)
		
		self.plotFitsRowsData.rotate(-90)
		self.plotFitsRowsData.setLogMode(False, log_mode)
		self.plotFitsRowsData.setFillLevel(fill_level)
		self.plotFitsRowsData.rotate(90)
		
		self.plotFitsColsData.setLogMode(False, log_mode)
		self.plotFitsColsData.setFillLevel(fill_level)
		
		lutHistog = self.lutHistog
		lutHistog.sigLevelChangeFinished.disconnect(self.onImageLevelsChanged)
		self.image.setImage(image_array)
		self.lutHistog.setLevels(*lutHistog_levels)
		self.lutHistog.axis.setLogMode(log_mode)
		lutHistog.sigLevelChangeFinished.connect(self.onImageLevelsChanged)
	
	def onAutoScaleChanged(self, state):
		
		auto_scale_mode = (state & Qt.Checked)
		log_mode = self.chkLogScale.isChecked()
		
		image_array = self.image_array
		if log_mode:
			image_array = log10(self.image_array)
		
		if auto_scale_mode:
			self.lutHistog.sigLevelChangeFinished.disconnect(
				self.onImageLevelsChanged)
			self.lutHistog.autoHistogramRange()
			self.lutHistog.setLevels(image_array.min(), image_array.max())
			self.lutHistog.sigLevelChangeFinished.connect(
				self.onImageLevelsChanged)
	
	def onImageLevelsChanged(self, lutHistog):
	
		self.chkAutoScale.setChecked(False)
		
# 		log_mode = self.chkLogScale.isChecked()
# 		
# 		lutHistog_levels = lutHistog.getLevels()
# 		print "lutHistog_levels:", lutHistog_levels
# 		# if lutHistog.axis.logMode:
# 		if log_mode:
# 			lutHistog_levels = 10 ** array(lutHistog_levels)
# 			print "[LOG] lutHistog_levels:", lutHistog_levels
# 		
# 		# self.image
# 		print "getLevels():", lutHistog.imageItem().getLevels()
# 		lutHistog.imageItem().setLevels(lutHistog_levels)
# 		print "[AFTER] getLevels():", lutHistog.imageItem().getLevels()
# 		# lutHistog.update()

	def onPauseFitsParamsUpdatesChanged(self, state):

		if self.data_pathnames:
			self.loadDataFile()
	
	def keyPressEvent(self, keyEvent):
		
		if ((
				(keyEvent.key() == Qt.Key_Q) and 
				(keyEvent.modifiers() == Qt.ControlModifier)
				) or 
			(
				(keyEvent.key() == Qt.Key_W) and 
				(keyEvent.modifiers() == Qt.ControlModifier)
				)):
			print "QUIT"
			QCoreApplication.instance().quit()
	
	def onMouseMoved(self, event):
		
		# pos = event.pos()
		pos = [event.x(), event.y()]
		
		if self.image.sceneBoundingRect().contains(event):
		
			# self.image.setCursor(Qt.CrossCursor)
			view_pos = self.imv.mapToView(event)
# 			image_pos = self.imv.mapFromViewToItem(
# 				self.image, view_pos)
			image_pos = self.image.mapFromScene(event)
			# print "\nEvent Pos:", event
			# print "\tView Pos:", view_pos
			# print "\tImage Pos:", image_pos
			# pos = [event.x(), event.y()]
			# pos = [view_pos.x(), view_pos.y()]
			pos = [image_pos.x(), image_pos.y()]
			# coords = tuple([int(round(coord)) for coord in pos])
			coords = tuple([int(floor(coord)) for coord in pos])
			# coord_text = "({0:d}, {1:d})".format(*coords)
			coord_text = "(r:{1:d}, c:{0:d})".format(*coords)
			coord_text += "\nIntemsity:{0:0,.0f}".format(
				self.image_array[coords[1], coords[0]],
				)
			self.image.setToolTip(coord_text)
			
# 			if self.image_cursor_label is None:
# 				label = pg.TextItem(coord_text, anchor=(1,0), fill=(0,0,0,80))
# 				label.setParentItem(self.image)
# 				# label.setPos(*coords)
# 				# label.setPos(event)
# 				# label.setPos(view_pos)
# 				label.setPos(image_pos)
# 				self.image_cursor_label = label
# 			else:
# 				label = self.image_cursor_label
# 				label.setText(coord_text)
# 				# label.setPos(*coords)
# 				# label.setPos(event)
# 				# label.setPos(view_pos)
# 				label.setPos(image_pos)
# 			anchor_point = [1, 0]
# 			if not label.viewRect().contains(
# 					label.boundingRect().topLeft() ):
# 				anchor_point[0] = 0
# 			if not label.viewRect().contains(
# 					label.boundingRect().bottomRight() ):
# 				anchor_point[1] = 1
# 			label.setAnchor(anchor_point)
# 		else:
# 			if self.image_cursor_label is not None:
# 				self.image_cursor_label.scene().removeItem(
# 					self.image_cursor_label)
# 				self.image_cursor_label = None
	
	def selectDataFile(self):
		
		# data_pathname = "{}Log_NiFe_00d_1340-060.fits".format(self.data_directory)
# 		data_pathname = "{}NiFe_00d_1340-060.fits".format(self.data_directory)
# 		self.data_pathnames = [data_pathname]
# 		self.data_frame_num = 1
		
# 		png_filename = "{}NiFe_00d_1340-060.png".format(self.data_directory)
# 		self.image_array = array(Image.open(png_filename))
# 		# self.image_array = array(Image.open(png_filename).convert("L"))
# 		# print "self.image_array:", self.image_array
# 		# print "self.image_array.shape:", self.image_array.shape
# 		# exit()
		
		# QFileDialog.setFileMode(QFileDialog.ExistingFile)
		selected_pathname = str(QFileDialog.getOpenFileName(
			self, 
			"Select data file", 
			self.working_directory, 
			# "FITS (*.fits);; Images (*.png *.jpg *.jpeg *.gif *.tiff);; AI files (*-AI.txt)")
			"FITS, PNG, AI files (*.fits *.png *-AI.txt)")
			)
		
		# print "selected_pathname:", selected_pathname
		
		self.raise_()
		self.activateWindow()
		
		data_frame_num = self.data_frame_num
		selected_pathname_parts = selected_pathname.split('/')
		data_path = "/".join(selected_pathname_parts[:-1])
		self.working_directory = data_path
		
		if selected_pathname.endswith("-AI.txt"):
		
# 			data_file = open(selected_pathname, 'r')
# 			file_line = data_file.readline().rstrip('\r\n')
# 			imgscan_header_linenum = 0
# 			while not file_line.startswith("Frame"):
# 				file_line = data_file.readline().rstrip('\r\n')
# 				imgscan_header_linenum++
# 			data_file.close()
			
			with open(selected_pathname, 'r') as data_file:
				for (header_linenum, file_line) in enumerate(data_file):
					# print (header_linenum, file_line)
					if file_line.startswith("Frame"):
						break
					if file_line[0].isdigit():
						header_linenum -= 1
			
			# check for missing header line
			# print "header_linenum:", header_linenum

			# header_linenum = 9
			imgscan_data = pd.read_table(
				selected_pathname,
				delimiter='\t',
				header=header_linenum,
				skip_blank_lines=False,
				)
			# imgscan_num_rows = len(imgscan_data)

			self.data_pathnames = ["{0:s}/WinView CCD/{1:s}.fits".format(
				data_path,
				filename,
				) for filename in imgscan_data['PNG Image filename'] ]
			
			if (self.data_frame_num > len(self.data_pathnames) ):
				# self.data_frame_num = len(self.data_pathnames)
				data_frame_num = len(self.data_pathnames)
		
		else:
			
			self.data_pathnames = [selected_pathname]
			# self.data_frame_num = 1
			data_frame_num = 1
		
		self.editDataFilename.setText(selected_pathname)
		self.spinDataFrameNum.setMinimum(1)
		self.spinDataFrameNum.setMaximum(len(self.data_pathnames))
		# self.spinDataFrameNum.setValue(self.data_frame_num)
		self.spinDataFrameNum.setValue(data_frame_num)
		
		self.loadDataFile(fullView=True)
	
	def loadDataFile(self, fullView=False):
		
		data_pathname = self.data_pathnames[self.data_frame_num - 1]
		
		# print "data_pathname:", data_pathname
		
		data_pathname_parts = data_pathname.split('/')
		data_path = " / ".join(data_pathname_parts[:-1])
		data_filename = data_pathname_parts[-1]
		
		self.txtImageTitle.setText(data_filename)
		self.txtImageSubtitle.setText(data_path)
		
		if data_filename.lower().endswith(".fits"):
		
			# hdu_pair    = load_image(data_pathname)
			# hdu_header  = hdu_pair[0]
			# hdu_image   = hdu_pair[-1]
			
			hdulist = fits.open(data_pathname)
			ccd_image = CcdImageFromFITS(
				hdulist,
				# offset_diffractometer = offset_angles
				)

			# self.image_array = hdu_image.data
			self.image_array = ccd_image.data
			self.imv.invertY(False)		# Need to invert if image is not FITS
			
			if self.cbPauseParamUpdates.isChecked():
				self.updateFitsParameters(None)
			else:
				self.updateFitsParameters(ccd_image)
		
		else:
		
			self.image_array = array(Image.open(data_pathname))
			self.imv.invertY(True)		# Need to invert if image is not FITS
			
			self.updateFitsParameters(None)
		
		image_array = self.image_array
		log_mode = self.chkLogScale.isChecked()
		if log_mode:
			image_array = log10(self.image_array)
		(imgRows, imgCols) = image_array.shape
			
		self.image.setImage(image_array)
		self.image.setLevels(self.lutHistog.getLevels())
		self.linRegRows.setBounds((0, imgRows))
		self.linRegCols.setBounds((0, imgCols))
		fill_level = image_array.min()
		self.plotFitsRowsData.setFillLevel(fill_level)
		self.plotFitsColsData.setFillLevel(fill_level)
		if fullView:
			self.imv.autoRange()
		self.onLinRegRowsChanged(self.linRegRows)
		self.onLinRegColsChanged(self.linRegCols)
	
	def updateDataFrame(self, value):
		
		self.data_frame_num = int(value)
		self.loadDataFile()
	
	def gotoFirstDataFrame(self):
		
		self.spinDataFrameNum.setValue(self.spinDataFrameNum.minimum())
	
	def gotoPrevDataFrame(self):
		
		self.spinDataFrameNum.stepDown()
	
	def gotoNextDataFrame(self):
		
		self.spinDataFrameNum.stepUp()
	
	def gotoLastDataFrame(self):
		
		self.spinDataFrameNum.setValue(self.spinDataFrameNum.maximum())
	
	def updateFitsParameters(self, ccdImage=None):
		
		self.model_fits_params_left.loadParams(ccdImage)
		self.model_fits_params_right.loadParams(ccdImage)

def main():
	app = QApplication(sys.argv)
# 	print "QStyleFactory.keys():", [str(s) for s in QStyleFactory.keys()]
# 	print "app.style():", app.style(), type(app.style())
# 	print QApplication.style().metaObject().className(), "\n"
# 	for key in QStyleFactory.keys():
# 		st = QStyleFactory.create(key)
# 		print key, st.metaObject().className(), type(app.style())
	# app.setStyle("Windows")
	mainWin = FitsViewerApp()
	mainWin.show()
	mainWin.raise_()
	# mainWin.activateWindow()
	app.exec_()

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-

import sip
sip.setapi('QVariant', 2)	# Call this before referencing QtCore
sip.setapi('QString', 2)	# Call this before referencing QtCore

# from PyQt4 import QtGui, QtCore
# from PyQt4.QtCore import Qt, QString, QVariant, QRect, QRectF, QSize, QPoint
from PyQt4.QtCore import Qt, QRect, QRectF, QSize, QPoint
from PyQt4.QtCore import QAbstractTableModel, QModelIndex
from PyQt4.QtGui import QApplication, QMainWindow, QTextEdit, QLabel, QStyle
from PyQt4.QtGui import QStyledItemDelegate, QStyleOptionViewItemV4
from PyQt4.QtGui import QTextDocument, QPushButton, QStyleOptionButton
from PyQt4.QtGui import QStyleFactory
# from PyQt4.QtGui import QCommonStyle, QCleanlooksStyle, QPlastiqueStyle
# from PyQt4.QtGui import QMacStyle, QWindowsStyle, QWindowsXPStyle

from datetime import datetime, date

from numpy import nan, float32, sqrt, array, isnan
import pandas as pd
import itertools as it

import sys
from als.milo.qimage import Diffractometer402, CcdImageFromFITS, Polarization

try:
    # _fromUtf8 = QString.fromUtf8
    pass
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QApplication.translate(context, text, disambig)

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class fitsParamsModel(QAbstractTableModel):
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	def __init__(self, parent=None, ccdImage=None):
		super(fitsParamsModel, self).__init__(parent)
		
		self.numRows = 0
		self.numCols = 3
		
		labels = []
		values = []
		units = []
		
		(
			self.labels_col, 
			self.values_col, 
			self.units_col, 
			) = range(self.numCols)
		
		self.df = pd.DataFrame(
			{
				"label": labels, 
				"value": values, 
				"unit": units, 
				}, 
			)
		self.df = self.df.reindex_axis([
			"label", 
			"value", 
			"unit", 
			], axis=1)
		
		self.loadParams(ccdImage)
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def loadParams(self, ccdImage=None):
		
		self.ccdImage = ccdImage
		
		if ccdImage is None:
			self.df.loc[:, "value"] = nan
			self.dataChanged.emit(
				self.index(0, self.values_col), 
				self.index(len(self.df), self.values_col) )
			return()
		
		self.loadParamsFromCcdFits(ccdImage)
		self.dataChanged.emit(
			self.index(0, self.values_col), 
			self.index(len(self.df), self.values_col) )
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def clearParams(self):
		
		self.loadParams(ccdImage=None)
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def loadParamsFromCcdFits(self, ccdImage):
		
		return()
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def rowCount(self, parent=None):
		if (parent is None) or (parent == QModelIndex()):
			return(self.numRows)
		return(0)
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def columnCount(self, parent=None):
		if (parent is None) or (parent == QModelIndex()):
			return(self.numCols)
		return(0)
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def data(self, index, role = Qt.DisplayRole):
		if (role == Qt.DisplayRole):
			return self.df.iloc[index.row(), index.column()]
# 		if (role == Qt.TextAlignmentRole):
# 			if (index.column() == self.units_col):
# 				return Qt.AlignLeft
# 			else:
# 				return Qt.AlignRight
		return None

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class fitsParamsLeftColModel(fitsParamsModel):
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def __init__(self, parent=None, ccdImage=None):
		super(fitsParamsLeftColModel, self).__init__(parent, ccdImage)
		
		table_contents = array([
			["Exposure time", nan, "sec."], 
			[" ", nan, " "], 
			[" ", nan, " "], 
			["Energy", nan, "eV"], 
			[" ", nan, " "], 
			["Detector, <i>2&theta;</i>", nan, "deg."], 
			["(2D) Incidence, <i>&alpha;</i>", nan, "deg."], 
			["(2D) Exit, <i>&beta;</i>", nan, "deg."], 
			["Transverse, <i>&chi;</i>", nan, "deg."], 
			[" ", nan, " "], 
			["X", nan, "mm"], 
			["Y", nan, "mm"], 
			["Z", nan, "mm"], 
			[" ", nan, " "], 
			# ["q<span class='sub'>X</span>", nan, "nm<span class='sup'>-1</span>"], 
			["q<sub>X</sub>", nan, "nm<sup>-1</sup>"], 
			["q<sub>Y</sub>", nan, "nm<sup>-1</sup>"], 
			["q<sub>Z</sub>", nan, "nm<sup>-1</sup>"], 
			])
		
		(self.numRows, self.numCols) = table_contents.shape
		
# 		(
# 			labels_col, 
# 			values_col, 
# 			units_col, 
# 			) = range(self.numCols)
		
		labels = table_contents[:, self.labels_col]
		values = table_contents[:, self.values_col]
		units = table_contents[:, self.units_col]
		
		blankline = it.count(0)
		indices = [
			"expTime", 
			"blank_{0:d}".format(blankline.next()), 
			"blank_{0:d}".format(blankline.next()), 
			"energy", 
			"blank_{0:d}".format(blankline.next()), 
			"twotheta", 
			"incidence2D", 
			"exit2D", 
			"transverse", 
			"blank_{0:d}".format(blankline.next()), 
			"x", 
			"y", 
			"z", 
			"blank_{0:d}".format(blankline.next()), 
			"qx", 
			"qy", 
			"qz", 
			]
		
		self.df = pd.DataFrame(
			{
				"label": labels, 
				"value": values, 
				"unit": units, 
				}, 
			index = pd.Index(indices), 
			)
		self.df = self.df.reindex_axis([
			"label", 
			"value", 
			"unit", 
			], axis=1)
		
		# print self.df["values"]
		# self.df["values"] = self.df["values"].astype(float32)
		# print self.df["values"]
		# print self.df
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def loadParamsFromCcdFits(self, ccdImage):
		
		diffractometer = ccdImage._diffractometer
		primary_hdu = ccdImage.hdulist[0].header
		qimage_df = ccdImage.qvalues_df()
		(num_rows, num_cols) = ccdImage.data.shape
		center_pixel = (
# 			(qimage_df["row"] == int(num_rows*(460./512.))) & 
			(qimage_df["row"] == int(num_rows - num_rows*(460./512.))) & 
			(qimage_df["col"] == int(num_cols/2.)) )
		
		self.df.loc["expTime", "value"] = ccdImage.exposure_time
		self.df.loc["energy", "value"] = diffractometer.energy
		self.df.loc["twotheta", "value"] = diffractometer.twotheta
		self.df.loc["incidence2D", "value"] = diffractometer.incidence
		self.df.loc["exit2D", "value"] = diffractometer.exit_angle
		self.df.loc["transverse", "value"] = diffractometer.transverse
		self.df.loc["x", "value"] = primary_hdu['X Position']
		self.df.loc["y", "value"] = primary_hdu['Y Position']
		self.df.loc["z", "value"] = primary_hdu['Z Position']
		self.df.loc["qx", "value"] = qimage_df.loc[center_pixel, "Qx"].values
		self.df.loc["qy", "value"] = qimage_df.loc[center_pixel, "Qy"].values
		self.df.loc["qz", "value"] = qimage_df.loc[center_pixel, "Qz"].values
		
		# print self.df["values"]
		# self.df["values"] = self.df["values"].astype(float32)
		# print self.df["values"]
		# print self.df

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class fitsParamsRightColModel(fitsParamsModel):
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def __init__(self, parent=None, ccdImage=None):
		super(fitsParamsRightColModel, self).__init__(parent, ccdImage)
		
		table_contents = array([
			["Date", nan, " "], 
			["Timestamp", nan, " "], 
			[" ", nan, " "], 
			["Polarization", nan, " "], 
			[" ", nan, " "], 
			["Bottom Seal", nan, "deg."], 
			["Top Seal", nan, "deg."], 
			["Top Seal <i>offset</i>", nan, "deg."], 
			["Flip", nan, "deg."], 
			["Flip <i>offset</i>", nan, "deg."], 
			[" ", nan, " "], 
			["Temperature A", nan, "K"], 
			["Temperature B", nan, "K"], 
			[" ", nan, " "], 
			["|q|", nan, "nm<sup>-1</sup>"], 
			["q<sub>In-Plane</sub>", nan, "nm<sup>-1</sup>"], 
			["q<sub>Out-of-Plane</sub>", nan, "nm<sup>-1</sup>"], 
			])
		
		(self.numRows, self.numCols) = table_contents.shape
		
# 		(
# 			labels_col, 
# 			values_col, 
# 			units_col, 
# 			) = range(self.numCols)
		
		labels = table_contents[:, self.labels_col]
		values = table_contents[:, self.values_col]
		units = table_contents[:, self.units_col]
		
		blankline = it.count(0)
		indices = [
			"dateStamp", 
			"timeStamp".format(blankline.next()), 
			"blank_{0:d}".format(blankline.next()),
			"polarization".format(blankline.next()), 
			"blank_{0:d}".format(blankline.next()), 
			"bottom", 
			"top", 
			"topOffset", 
			"flip", 
			"flipOffset", 
			"blank_{0:d}".format(blankline.next()), 
			"tempA", 
			"tempB", 
			"blank_{0:d}".format(blankline.next()), 
			"qmag", 
			"qip", 
			"qop", 
			]
		
		self.df = pd.DataFrame(
			{
				"label": labels, 
				"value": values, 
				"unit": units, 
				}, 
			index = pd.Index(indices), 
			)
		self.df = self.df.reindex_axis([
			"label", 
			"value", 
			"unit", 
			], axis=1)
		
		# print self.df["values"]
		# self.df["values"] = self.df["values"].astype(float32)
		# print self.df["values"]
		# print self.df
		
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def loadParamsFromCcdFits(self, ccdImage):
		
		diffractometer = ccdImage._diffractometer
		primary_hdu = ccdImage.hdulist[0].header
		qimage_df = ccdImage.qvalues_df()
		(num_rows, num_cols) = ccdImage.data.shape
		center_pixel = (
			(qimage_df["row"] == int(num_rows*(460./512.))) & 
			(qimage_df["col"] == int(num_cols/2.)) )
			
		value = primary_hdu.get('DATE', None)
		if value is not None:
			if 'T' in value:
				image_date = datetime.strptime(value, 
					"%Y-%m-%dT%H:%M:%S").date()
				image_time = datetime.strptime(value, 
					"%Y-%m-%dT%H:%M:%S").time()
			else:
				image_date = datetime.strptime(value, "%Y-%m-%d").date()
				image_time = datetime.strptime(value, "%H:%M:%S").time()
		else:
			value = primary_hdu.get('DATETIME', None)
			if value is not None:
				image_date = datetime.strptime(value, 
					"%Y/%m/%d %H:%M:%S").date()
				image_time = datetime.strptime(value, 
					"%Y/%m/%d %H:%M:%S").time()
			else:
				image_date = datetime.strptime("1970-01-01", "%Y-%m-%d").date()
				image_time = datetime.strptime("1970-01-01", "%H:%M:%S").time()
		
		pol = diffractometer.polarization
		pol_state_text = dict({
			Polarization.UNDEFINED: "N/A",
			Polarization.CIRCULAR:	"Circ.",
			Polarization.LINEAR:	"Lin.", 
			})
		pol_value_text = dict({
			Polarization.UNDEFINED: nan,
			Polarization.CIRCULAR:	pol.circular_degree,
			Polarization.LINEAR:	pol.linear_angle, 
			})
		
		for key_entry in ('TempCtrlrA', 'Temperature A', 
						  'Lakeshore Temp Controller A', ):
			value = primary_hdu.get(key_entry, None)
			if value is not None:
				temperatureA = value
				break
		else:
			temperatureA = 298		# Kelvin
		
		for key_entry in ('TempCtrlrB', 'Temperature B', 
						  'Lakeshore Temp Controller B', ):
			value = primary_hdu.get(key_entry, None)
			if value is not None:
				temperatureB = value
				break
		else:
			temperatureB = 298		# Kelvin
		
		qxy = sqrt(qimage_df.loc[center_pixel, "Qx"].values**2 + 
				qimage_df.loc[center_pixel, "Qy"].values**2)
		
		qmag = sqrt(qimage_df.loc[center_pixel, "Qz"].values**2 + qxy**2)
		
		self.df.loc["polarization", "unit"] = pol_state_text[pol.state]
			
		self.df.loc["dateStamp", "value"] = image_date
		self.df.loc["timeStamp", "value"] = image_time
		self.df.loc["polarization", "value"] = pol_value_text[pol.state]
		self.df.loc["bottom", "value"] = diffractometer.bottom_angle
		self.df.loc["top", "value"] = diffractometer.top_angle
		self.df.loc["topOffset", "value"] = diffractometer.offset_top
		self.df.loc["flip", "value"] = diffractometer.flip_angle
		self.df.loc["flipOffset", "value"] = diffractometer.offset_flip
		self.df.loc["tempA", "value"] = temperatureA
		self.df.loc["tempB", "value"] = temperatureB
		self.df.loc["qmag", "value"] = qmag
		self.df.loc["qip", "value"] = qxy
		self.df.loc["qop", "value"] = qimage_df.loc[center_pixel, "Qz"].values
		
		# print self.df["values"]
		# self.df["values"] = self.df["values"].astype(float32)
		# print self.df["values"]
		# print self.df
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
# 	def data(self, index, role = Qt.DisplayRole):
# 		if (role == Qt.DisplayRole):
# 			value = super(fitsParamsRightColModel, self).data(index, role)
# 			if ( (self.df.index[index.row()] == "dateStamp")
# 					and (index.column() == self.values_col)
# 					):
# 				print "dateStamp"
# 				# print value.strftime("%A, %B %d, %Y")
# 				print value.strftime("%c")
# 				return value.strftime("%c")
# 				return value.strftime("%A, %B %d, %Y")
# 			if ( (self.df.index[index.row()] == "timeStamp")
# 					and (index.column() == self.values_col)
# 					):
# 				print "timeStamp"
# 				print value.strftime("%H : %M : %S . %f")
# 				return value.strftime("%H : %M : %S . %f")
# 			return value
# 		return None
	
	# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	def data(self, index, role = Qt.DisplayRole):
		if (role == Qt.DisplayRole):
			value = super(fitsParamsRightColModel, self).data(index, role)
			if (self.df.index[index.row()] == "dateStamp"):
				return self.df.iloc[index.row(), self.values_col]
			if (self.df.index[index.row()] == "timeStamp"):
				return self.df.iloc[index.row(), self.values_col]
			return value
		return None

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtLabelDelegate(QStyledItemDelegate):
	def __init__(self, parent=None):
		super(QStyledItemDelegate, self).__init__(parent)
	
	def paint (self, painter, option, index):
		return
# 		# item = QLabel(QString(index.data()), self)
# 		# item = QLabel(str(index.data()), self)
# 		item = QLabel(index.data().toString(), self.parent())
# 		item.setDisabled(True)
# 		item.setAutoFillBackground(True)
# 		# item.render(painter)
# 		item.drawContents(painter)
	
	# update sizeHint() ?

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtHtmlDelegate(QStyledItemDelegate):
	def __init__(self, parent=None):
		super(QStyledItemDelegate, self).__init__(parent)
	
	def paint(self, painter, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		
# 		print "option.displayAlignment:"
# 		print "\tLeft :", (option.displayAlignment & Qt.AlignLeft)
# 		print "\tRight:", (option.displayAlignment & Qt.AlignRight)
		
		# if (option.displayAlignment & Qt.AlignLeft): print "Left"
		# if (option.displayAlignment & Qt.AlignRight): print "Right"
		
		painter.save()
		
		doc = QTextDocument()
		doc.setHtml(options.text)
		
		options.text = ""
		options.widget.style().drawControl(
			QStyle.CE_ItemViewItem, options, painter)
		
		# Assuming .alignment() & Qt.AlignVCenter
		doc_size = self.sizeHint(option, index)
		height_offset = (options.rect.height() - doc_size.height()) / 2.
# 		print "\n", index.row(), ",", index.column(), ":"
# 		print "options.rect.height():", options.rect.height()
# 		print "doc_size.height():", doc_size.height()
# 		print "height_offset:", height_offset
# 		print "doc_size.width():", doc_size.width()
		height_offset = max(0, height_offset)
		painter.translate(
			options.rect.left(), 
			options.rect.top() + height_offset)
		clip = QRectF(0, 0, options.rect.width(), options.rect.height())
		doc.drawContents(painter, clip)
		
		painter.restore()
	
	def sizeHint(self, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		
		doc = QTextDocument()
		doc.setHtml(options.text)
		# doc.setTextWidth(options.rect.width())	# For multiline text
		return QSize(doc.idealWidth(), doc.size().height())

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtFloatDelegate(QStyledItemDelegate):
	def __init__(self, parent=None, decimals=6):
		super(QStyledItemDelegate, self).__init__(parent)
		self.decimals = decimals
	
# 	def paint(self, painter, option, index):
# 		painter.save()
# 		
# 		# value = float(index.data().toPyObject())
# 		value = float(index.data())
# 		# painter.drawText(
# 		# 	option.rect, 
# 		# 	Qt.AlignRight & Qt.AlignVCenter, 
# 		# 	"{0:0.{1:d}f}".format(value, self.decimals))
# 		
# 		# Assuming .alignment() & Qt.AlignRight & Qt.AlignVCenter
# 		doc_size = self.sizeHint(option, index)
# 		height_offset = (option.rect.height() - doc_size.height()) / 2.
# 		print "\n", index.row(), ",", index.column(), ":"
# 		print "option.rect.height():", option.rect.height()
# 		print "doc_size.height():", doc_size.height()
# 		print "height_offset:", height_offset
# 		print "doc_size.width():", doc_size.width()
# 		height_offset = max(0, height_offset)
# 		height_offset = 0
# 		painter.translate(
# 			# option.rect.right() - doc_size.width(), 
# 			option.rect.left(), 
# 			# option.rect.left() + doc_size.width()/2., 
# 			option.rect.top() + height_offset)
# 		painter.drawText(
# 			option.rect, 
# 			# Qt.AlignRight & Qt.AlignVCenter, 
# 			Qt.AlignLeft & Qt.AlignTop, 
# 			"{0:0.{1:d}f}".format(value, self.decimals))
# 		
# 		painter.restore()
	
	def paint(self, painter, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		
		painter.save()
		
		doc = QTextDocument()
		# value = float(index.data().toPyObject())
		value = float(index.data())
		doc.setPlainText("{0:0.{1:d}f}".format(value, self.decimals))
		if isnan(value):
			doc.setPlainText("")
		
		options.text = ""
		options.widget.style().drawControl(
			QStyle.CE_ItemViewItem, options, painter)
		
		# Assuming .alignment() & Qt.AlignRight & Qt.AlignVCenter
		doc_size = self.sizeHint(option, index)
		height_offset = (options.rect.height() - doc_size.height()) / 2.
# 		print "\n", index.row(), ",", index.column(), ":"
# 		print "options.rect.height():", options.rect.height()
# 		print "doc_size.height():", doc_size.height()
# 		print "height_offset:", height_offset
# 		print "doc_size.width():", doc_size.width()
		height_offset = max(0, height_offset)
		painter.translate(
			options.rect.right() - doc_size.width(), 
			options.rect.top() + height_offset)
		clip = QRectF(0, 0, options.rect.width(), options.rect.height())
		doc.drawContents(painter, clip)
		
		painter.restore()
	
	def sizeHint(self, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data().toPyObject()
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data()
		
		doc = QTextDocument()
		# value = float(index.data().toPyObject())
		value = float(index.data())
		doc.setPlainText("{0:0.{1:d}f}".format(value, self.decimals))
		# doc.setTextWidth(options.rect.width())	# For multiline text
		return QSize(doc.idealWidth(), doc.size().height())

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtButtonDelegate(QStyledItemDelegate):
	def __init__(self, parent=None, decimals=6):
		super(QStyledItemDelegate, self).__init__(parent)
	
	def paint(self, painter, option, index):
		# options = QStyleOptionViewItemV4(option)
		# self.initStyleOption(options, index)
		
		painter.save()
		
		doc = QPushButton(index.data(), self.parent() )
		
		# options.text = ""
		# options.widget.style().drawControl(
		# 	QStyle.CE_PushButton, options, painter)
		btn_option = QStyleOptionButton()
		# btn_option.initFrom(option)
		# print "btn_option.state:", type(btn_option.state), btn_option.state
		# print "QStyle.State_Enabled:", type(QStyle.State_Enabled), QStyle.State_Enabled
		# print "QStyle.State_Raised:", type(QStyle.State_Raised), QStyle.State_Raised
		print "btn_option.features:", btn_option.features
		# btn_option.features = QStyleOptionButton.None
		# btn_option.features = QStyleOptionButton.Flat
		btn_option.state = QStyle.State_Enabled or QStyle.State_Raised
		btn_option.state = QStyle.State_Raised
		btn_option.rect = option.rect
		btn_option.text = index.data()
		doc.resize(doc.minimumSizeHint())
		
		# Assuming .alignment() & Qt.AlignCenter
		doc_size = self.sizeHint(option, index)
		btn_option.rect = QRect(
			QPoint(option.rect.left(), option.rect.top()), doc_size)
		btn_option.rect = QRect(
			QPoint(option.rect.left()+5, option.rect.top()+5), 
			QSize(doc_size.width()-10, doc_size.height()-10) )
		btn_option.rect = option.rect.adjusted(1, 1, -1, -1)
		btn_option.rect = option.rect.adjusted(2, 2, -2, -2)
		btn_option.rect = option.rect.adjusted(15, 5, -15, -5)
		height_offset = (option.rect.height() - doc_size.height()) / 2.
		width_offset = (option.rect.width() - doc_size.width()) / 2.
		print "\n", index.row(), ",", index.column(), ":"
		print "option.rect.height():", option.rect.height()
		print "doc_size.height():", doc_size.height()
		print "height_offset:", height_offset
		print "doc_size.width():", doc_size.width()
		height_offset = max(0, height_offset)
		width_offset = max(0, width_offset)
# 		painter.translate(
# 			option.rect.left() + width_offset, 
# 			option.rect.top() + height_offset)
# 		clip = QRectF(0, 0, option.rect.width(), option.rect.height())
		# doc.drawContents(painter, clip)
		# doc.style().drawControl(
		QApplication.style().drawControl(
			QStyle.CE_PushButton, btn_option, painter)
		
		painter.restore()
	
	def sizeHint(self, option, index):
		# options = QStyleOptionViewItemV4(option)
		# self.initStyleOption(options, index)
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data().toPyObject()
		print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data()
		
		doc = QPushButton(index.data())
		print "doc.sizeHint():", doc.sizeHint()
		print "doc.minimumSizeHint():", doc.minimumSizeHint()
		doc.resize(doc.minimumSizeHint())
		print "doc.size():", doc.size()
		print "doc.rect():", doc.rect().left(), doc.rect().top(), doc.rect().right(), doc.rect().bottom()
		print "doc.contentsRect():", doc.contentsRect().left(), doc.contentsRect().top(), doc.contentsRect().right(), doc.contentsRect().bottom()
		print "doc.contentsMargins():", doc.contentsMargins().left(), doc.contentsMargins().top(), doc.contentsMargins().right(), doc.contentsMargins().bottom()
		# doc.setTextWidth(options.rect.width())	# For multiline text
		# return QSize(doc.idealWidth(), doc.size().height())
		return doc.size()

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtDateDelegate(QStyledItemDelegate):
	def __init__(self, parent=None):
		super(QStyledItemDelegate, self).__init__(parent)
		
	def paint(self, painter, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		
		painter.save()
		
		doc = QTextDocument()
		value = index.data()
		try:
			doc.setPlainText(value.strftime("%A, %B %d, %Y"))
		except:
			doc.setPlainText("")
		
		options.text = ""
		options.widget.style().drawControl(
			QStyle.CE_ItemViewItem, options, painter)
		
		# Assuming .alignment() & Qt.AlignLeft & Qt.AlignVCenter
		doc_size = self.sizeHint(option, index)
		height_offset = (options.rect.height() - doc_size.height()) / 2.
# 		print "\n", index.row(), ",", index.column(), ":"
# 		print "options.rect.height():", options.rect.height()
# 		print "doc_size.height():", doc_size.height()
# 		print "height_offset:", height_offset
# 		print "doc_size.width():", doc_size.width()
		height_offset = max(0, height_offset)
		painter.translate(
			# options.rect.right() - doc_size.width(), 
			options.rect.left(), 
			options.rect.top() + height_offset)
		clip = QRectF(0, 0, options.rect.width(), options.rect.height())
		doc.drawContents(painter, clip)
		
		painter.restore()
	
	def sizeHint(self, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data().toPyObject()
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data()
		
		doc = QTextDocument()
		value = index.data()
		try:
			doc.setPlainText(value.strftime("%A, %B %d, %Y"))
		except:
			doc.setPlainText("")
		# doc.setTextWidth(options.rect.width())	# For multiline text
		return QSize(doc.idealWidth(), doc.size().height())

# ----------------------------------------------------------------
# ----------------------------------------------------------------
class qtTimeDelegate(QStyledItemDelegate):
	def __init__(self, parent=None):
		super(QStyledItemDelegate, self).__init__(parent)
		
	def paint(self, painter, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		
		painter.save()
		
		doc = QTextDocument()
		value = index.data()
		try:
			doc.setPlainText("Time stamp: " + value.strftime("%H:%M:%S"))
		except:
			doc.setPlainText("")
		
		options.text = ""
		options.widget.style().drawControl(
			QStyle.CE_ItemViewItem, options, painter)
		
		# Assuming .alignment() & Qt.AlignLeft & Qt.AlignVCenter
		doc_size = self.sizeHint(option, index)
		height_offset = (options.rect.height() - doc_size.height()) / 2.
# 		print "\n", index.row(), ",", index.column(), ":"
# 		print "options.rect.height():", options.rect.height()
# 		print "doc_size.height():", doc_size.height()
# 		print "height_offset:", height_offset
# 		print "doc_size.width():", doc_size.width()
		height_offset = max(0, height_offset)
		painter.translate(
			# options.rect.right() - doc_size.width(), 
			options.rect.left(), 
			options.rect.top() + height_offset)
		clip = QRectF(0, 0, options.rect.width(), options.rect.height())
		doc.drawContents(painter, clip)
		
		painter.restore()
	
	def sizeHint(self, option, index):
		options = QStyleOptionViewItemV4(option)
		self.initStyleOption(options, index)
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data().toPyObject()
		# print "index ({0:d}, {1:d}): data =".format(index.row(), index.column()), index.data()
		
		doc = QTextDocument()
		value = index.data()
		try:
			doc.setPlainText("Time stamp: " + value.strftime("%H:%M:%S"))
		except:
			doc.setPlainText("")
		# doc.setTextWidth(options.rect.width())	# For multiline text
		return QSize(doc.idealWidth(), doc.size().height())

# ----------------------------------------------------------------
# ----------------------------------------------------------------


#!/usr/bin/env python

"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

# Non-qualified import of `versioneer`; local file is intended
import versioneer

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ALS.Liam',  # Required

    version=versioneer.get_version(),  # Required

    cmdclass=versioneer.get_cmdclass(),  # Required

    description='Visualize CCD data from ALS BL 4.0.2 diffractometer',  # Required

    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',

    url='https://bitbucket.org/berkeleylab/als.liam/src/master/',  # Optional

    author='Padraic Shafer',  # Optional
    author_email='pshafer@lbl.gov',  # Optional

    classifiers=[  # Optional

        'Development Status :: 4 - Beta',

        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Physics',

        # 'License :: OSI Approved :: BSD License :: BSD-3-Clause-LBNL',
        'License :: OSI Approved :: BSD License', # 'BSD-3-Clause-LBNL'

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',

        'Operating System :: OS Independent',
        'Environment :: Console',
    ],

    keywords='CCD RSXD x-ray ALS BL4.0.2 diffraction',  # Optional

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required

    install_requires=[
        'PkgScript',
        'ALS.Milo',
        'pyqtgraph',
        ],  # Optional

    extras_require={  # Optional
        # 'dev': ['check-manifest'],
        # 'test': ['coverage'],
    },

    include_package_data=True,

    entry_points={  # Optional
        'console_scripts': [
            'fitsViewer=als.liam.fitsViewer:main',
            # 'liam=als.liam.fitsViewer:main',
        ],
    },
)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'fitsViewer.ui'
#
# Created by: PyQt4 UI code generator 4.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1182, 756)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.widget = QtGui.QWidget(self.centralwidget)
        self.widget.setGeometry(QtCore.QRect(10, 10, 1161, 731))
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.vlDataView = QtGui.QVBoxLayout()
        self.vlDataView.setSpacing(5)
        self.vlDataView.setObjectName(_fromUtf8("vlDataView"))
        self.glFitsImage = GraphicsLayoutWidget(self.widget)
        self.glFitsImage.setObjectName(_fromUtf8("glFitsImage"))
        self.vlDataView.addWidget(self.glFitsImage)
        self.grdlDataButtons = QtGui.QGridLayout()
        self.grdlDataButtons.setHorizontalSpacing(5)
        self.grdlDataButtons.setVerticalSpacing(0)
        self.grdlDataButtons.setObjectName(_fromUtf8("grdlDataButtons"))
        self.btnFirstDataFrame = QtGui.QPushButton(self.widget)
        self.btnFirstDataFrame.setObjectName(_fromUtf8("btnFirstDataFrame"))
        self.grdlDataButtons.addWidget(self.btnFirstDataFrame, 1, 4, 1, 1)
        self.btnPrevDataFrame = QtGui.QPushButton(self.widget)
        self.btnPrevDataFrame.setObjectName(_fromUtf8("btnPrevDataFrame"))
        self.grdlDataButtons.addWidget(self.btnPrevDataFrame, 1, 5, 1, 1)
        self.btnSelectDataFile = QtGui.QPushButton(self.widget)
        self.btnSelectDataFile.setObjectName(_fromUtf8("btnSelectDataFile"))
        self.grdlDataButtons.addWidget(self.btnSelectDataFile, 0, 0, 1, 1)
        self.btnLastDataFrame = QtGui.QPushButton(self.widget)
        self.btnLastDataFrame.setObjectName(_fromUtf8("btnLastDataFrame"))
        self.grdlDataButtons.addWidget(self.btnLastDataFrame, 1, 7, 1, 1)
        self.spinDataFrameNum = QtGui.QSpinBox(self.widget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinDataFrameNum.sizePolicy().hasHeightForWidth())
        self.spinDataFrameNum.setSizePolicy(sizePolicy)
        self.spinDataFrameNum.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spinDataFrameNum.setObjectName(_fromUtf8("spinDataFrameNum"))
        self.grdlDataButtons.addWidget(self.spinDataFrameNum, 1, 3, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.grdlDataButtons.addItem(spacerItem, 1, 2, 1, 1)
        self.btnReloadDataFile = QtGui.QPushButton(self.widget)
        self.btnReloadDataFile.setObjectName(_fromUtf8("btnReloadDataFile"))
        self.grdlDataButtons.addWidget(self.btnReloadDataFile, 1, 0, 1, 1)
        self.btnNextDataFrame = QtGui.QPushButton(self.widget)
        self.btnNextDataFrame.setObjectName(_fromUtf8("btnNextDataFrame"))
        self.grdlDataButtons.addWidget(self.btnNextDataFrame, 1, 6, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.grdlDataButtons.addItem(spacerItem1, 1, 1, 1, 1)
        self.editDataFilename = QtGui.QLineEdit(self.widget)
        self.editDataFilename.setObjectName(_fromUtf8("editDataFilename"))
        self.grdlDataButtons.addWidget(self.editDataFilename, 0, 1, 1, 7)
        self.vlDataView.addLayout(self.grdlDataButtons)
        self.horizontalLayout.addLayout(self.vlDataView)
        self.vlFitsParams = QtGui.QVBoxLayout()
        self.vlFitsParams.setObjectName(_fromUtf8("vlFitsParams"))
        self.labelFitsParams = QtGui.QLabel(self.widget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelFitsParams.sizePolicy().hasHeightForWidth())
        self.labelFitsParams.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.labelFitsParams.setFont(font)
        self.labelFitsParams.setObjectName(_fromUtf8("labelFitsParams"))
        self.vlFitsParams.addWidget(self.labelFitsParams)
        self.hlFitsParamsCols = QtGui.QHBoxLayout()
        self.hlFitsParamsCols.setObjectName(_fromUtf8("hlFitsParamsCols"))
        self.viewFitsParamsLeft = QtGui.QTableView(self.widget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.viewFitsParamsLeft.sizePolicy().hasHeightForWidth())
        self.viewFitsParamsLeft.setSizePolicy(sizePolicy)
        self.viewFitsParamsLeft.setObjectName(_fromUtf8("viewFitsParamsLeft"))
        self.hlFitsParamsCols.addWidget(self.viewFitsParamsLeft)
        self.viewFitsParamsRight = QtGui.QTableView(self.widget)
        self.viewFitsParamsRight.setObjectName(_fromUtf8("viewFitsParamsRight"))
        self.hlFitsParamsCols.addWidget(self.viewFitsParamsRight)
        self.vlFitsParams.addLayout(self.hlFitsParamsCols)
        self.cbPauseParamUpdates = QtGui.QCheckBox(self.widget)
        self.cbPauseParamUpdates.setObjectName(_fromUtf8("cbPauseParamUpdates"))
        self.vlFitsParams.addWidget(self.cbPauseParamUpdates)
        spacerItem2 = QtGui.QSpacerItem(400, 150, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        self.vlFitsParams.addItem(spacerItem2)
        self.horizontalLayout.addLayout(self.vlFitsParams)
        self.horizontalLayout.setStretch(0, 7)
        self.horizontalLayout.setStretch(1, 3)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1182, 22))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.btnFirstDataFrame.setText(_translate("MainWindow", "<< First", None))
        self.btnPrevDataFrame.setText(_translate("MainWindow", "< Prev", None))
        self.btnSelectDataFile.setText(_translate("MainWindow", "Load data file", None))
        self.btnLastDataFrame.setText(_translate("MainWindow", "Last >>", None))
        self.btnReloadDataFile.setText(_translate("MainWindow", "Reload", None))
        self.btnNextDataFrame.setText(_translate("MainWindow", "Next >", None))
        self.labelFitsParams.setText(_translate("MainWindow", "Experiment parameters in FITS file", None))
        self.cbPauseParamUpdates.setText(_translate("MainWindow", "Pause updates of Experiment Parameters (for faster browsing)", None))

from pyqtgraph import GraphicsLayoutWidget
